package com.wjholden.nmap;

import java.io.File;
import java.io.IOException;
import java.util.StringTokenizer;

import android.content.Context;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.Message;

/**
 * @author William John Holden (wjholden@gmail.com)
 * Class created to centralize and more easily manage certain pieces of boilerplate
 * code, often code groups that are required more than once.
 * PMD recommended I make this a "singleton" so here goes.
 */
public final class NmapUtilities implements NmapConstants
{
	@SuppressWarnings("unused")
	private static final NmapUtilities FINAL_INSTANCE = new NmapUtilities();
	
	private NmapUtilities()
	{
		// deliberately empty
	}
	
	/**
	 * Removed from verifyInstallation for version 21.
	 * If temporary folder does not already exist, create it.
	 * @param c Context the application is executing in.
	 * @throws IOException
	 * @throws NameNotFoundException
	 * @return Absolute path of the temporary folder as String.
	 */
	public static String verifyTemporaryFolder(final Context context) throws IOException, NameNotFoundException
	{
		final File tmpdir = new File(context.getPackageManager().getApplicationInfo("com.wjholden.nmap", 0).dataDir + "/tmp/");
		NmapMain.ne.log("Checking if " + tmpdir.getAbsolutePath() + " exists...");
		if (tmpdir.exists())
		{
			NmapMain.ne.log(tmpdir.getAbsolutePath() + " exists.");
		}
		else
		{
			NmapMain.ne.log(tmpdir.getAbsolutePath() + " does not exist. Creating it...");
			if (tmpdir.mkdirs())
			{
				NmapMain.ne.log(tmpdir.getAbsolutePath() + " created.");
			}
			else
			{
				NmapMain.ne.log("Something went wrong - " + tmpdir.getAbsolutePath() + " could not be created.");
				
				final Message msg = Message.obtain();
				msg.obj = "Unable to create " + tmpdir.getAbsolutePath();
				msg.what = INSTALL_ERROR;
				NmapMain.handler.sendMessage(msg);
				//return null; // removing this line may introduce bugs where the program, wrongly, believes
				// that the tmpdir was successfully set when, in fact, it was not.
			}
		}
		return tmpdir.getAbsolutePath();
	}
	
	/**
	 * Mimics 'which' function to find 'su' command. Sets Nmap.sh and Nmap.canGetRoot.
	 * Originally, the program just looked for a hard-coded 'su' path, but CM6 has 'su'
	 * in a different location. Logic robusted to look everywhere in $PATH.
	 * Also sets shell, using Nmap.setSh(String).
	 * @since 0.4.1
	 * @return true if su found in $PATH.
	 */
	public static boolean checkRootPermissions()
	{
		// should now be fixed @version 0.4.1 to allow /system/xbin/su on CM5+
		final String path = System.getenv("PATH");
		File suTest;
		boolean hasFoundRoot = false;
		final StringTokenizer pathTokens = new StringTokenizer(path, ":");
		while (pathTokens.hasMoreTokens() && !hasFoundRoot)
		{
			final String token = pathTokens.nextToken();
			suTest = new File(token + "/su"); // sorry PMD, I can't think of a way to do this w/o creating new objects.
			hasFoundRoot = suTest.exists();
			if (hasFoundRoot)
			{
				NmapMain.setSh(suTest.getAbsolutePath());
				NmapMain.ne.log("su command found at " + NmapMain.sh + " will run with root permissions.");
			}
		}
		
		if (!hasFoundRoot)
		{
			NmapMain.setSh("sh");
			NmapMain.ne.log("su NOT found - will attempt to run without root permissions.");
		}
		
		return hasFoundRoot;
	}
	
	/**
	 * Find the data directory to store applications in. Normally will be of form /data/da
	 * @param c
	 * @return Absolute path of data directory as String.
	 */
	public static String getDataDirectory(final Context context)
	{
		String dataDir = null;
		try {
			dataDir = context.getPackageManager().getApplicationInfo("com.wjholden.nmap", 0).dataDir;
		} catch (NameNotFoundException e) {
			NmapMain.ne.log(e.getMessage());
		}
		return dataDir;
	}
	
	/**
	 * Based on whether 'su' is available or not this method sets the location to write
	 * compiled binaries for later execution.
	 * @return Absolute location to write binaries as String.
	 * @since 22 - UPDATE - now ALWAYS uses the {dataDir}/bin/ folder
	 * @param myDataDir - directory set by (above) getDataDirectory(Context)
	 */
	public static String findBinDir(final String myDataDir)
	{
		/*if (Nmap.canGetRoot)
			Nmap.setBinDir("/data/local/bin/");
		else
			Nmap.setBinDir(myDataDir + "/bin/"); */

		final String myBindir = myDataDir + "/bin/";
		
		NmapMain.ne.log("Set bindir to: " + myBindir);
		return myBindir;
	}
}
